#ifndef COTEXIO_H
#define COTEXIO_H

#include <qstring.h>
#include <Object.h>

#include "../Interfaces/TextureInterface.h"
#include "../Interfaces/CoTeXNode.h"
#include "../../ui/CoTeXConfigurationWidget.h"
#include "../Interfaces/PrimitiveManagerInterface.h"
#include "../Interfaces/OperatorManagerInterface.h"
#include "../Interfaces/CoTeXTreeInterface.h"

class CoTeXIOException {
public :
  
  CoTeXIOException (QString msg) : message (msg) {}
  inline QString getMessage () const { return message;}
  
private :
  QString message;
}; 

/// Tool for saving and loading ctx files
/**
 * The ctx format describes CoTeXTree and relative maps in a simple and extensive 
 * ASCII human readable file.
 * ctx saves CoTeX scenes (trees as nodes and textures, even if textures are not 
 * used, for keeping save the work on non-used textures).
 */
class CoTeXIO {
public :
  
  /**
   * Saves the tree in the ctx file "filename".
   * 
   * @param texMap all the map containing all the CoTeX textures.
   * @param tree the CoTeX tree to save.
   * @param filename the to write.
   */
  static void save (TextureInterface::TextureMap * texMap,
		    CoTeXTreeInterface * tree, 
		    const QString & filename);
  
/**
 * Load the file "filename", and return the CoTeXTree constructed. An exception 
 * CoTeXIOException is throwed when there are troubles with the file.
 *
 * @param tree the tree to fill with the ctx file data.
 * @param activeObject the object which will be textured by the loaded tree.
 * @param primitiveObjectMap the container for <code>Object</code> representing
 *                           the loaded primitives.
 * @param primitiveManagerMap the active managers used for generating primitives in 
 *                            CoTeX.
 * @param operatorManagerMap the active managers used for generating operators in 
 *                            CoTeX.
 * @param filename the file to load
 */
 static void load (CoTeXTreeInterface * tree,
		   Object * activeObject,
		   CoTeXConfigurationWidget * confWidget,
		   PrimitiveManagerInterface::PrimitiveObjectMap * primitiveObjectMap,
		   PrimitiveManagerInterface::PrimitiveManagerMap * primitiveManagerMap,
		   OperatorManagerInterface::OperatorManagerMap * operatorManagerMap,
		   const QString & filename);
private :
  
  static void loadTextures (CoTeXConfigurationWidget * confWidget,
			    QTextStream & stream);
  
  static void loadTree (CoTeXTreeInterface * tree, 
			Object * activeObject,
			CoTeXConfigurationWidget * confWidget,
			PrimitiveManagerInterface::PrimitiveObjectMap * primitiveObjectMap,
			PrimitiveManagerInterface::PrimitiveManagerMap * primitiveManagerMap,
			OperatorManagerInterface::OperatorManagerMap * operatorManagerMap,
			QTextStream & stream);
  
  static void saveNode (CoTeXNode * node, int & num, QTextStream & stream);
};

#endif // COTEXIO_H
