#ifndef BLOCK_H
#define BLOCK_H

#include <Vector3D.h>
#include "../Interfaces/PrimitiveInterface.h"
#include "../Interfaces/TextureInterface.h"
#include "../Textures/SimpleColorTexture.h"


/// A Block, specified by a center, and lengths along x, y, z.
class Block : public PrimitiveInterface{
public :

  /// General constructor for Block
  /**
   * @param n
   *        the name of the sphere
   * @param c
   *        the center of the <code>Block</code>.
   * @param dx 
   *        edge length along x
   * @param dy 
   *        edge length along y
   * @param dz 
   *        edge length along z
   * @param t
   *        the texture carried by this primitive.
   */
  Block (QString n, const Vector3D & c = Vector3D (0.0, 0.0, 0.0) , float dx = 1.0,
	 float dy = 1.0, float dz = 1.0, TextureInterface * t = DEFAULT_SIMPLECOLOR);
  
  Block (Block &b);
  virtual ~Block () {}
  
  inline float getDx () const { return d[0];}
  inline float getDy () const { return d[1];}
  inline float getDz () const { return d[2];}
  
  inline Vector3D getCenter () const { return center;}
  
  void setDx (float newDx); 
  void setDy (float newDy); 
  void setDz (float newDz); 
  
  inline void setCenter (Vector3D& newCenter) { center = newCenter;}
  
  inline Block& operator=(Block& block) {
    d = block.d;
    center = block.center;
    return (*this);
  }

  BBoxMinMax getBBoxMinMax ();

  // *******************
  // CoTeXNode Interface
  // *******************
  
  /// F-Rep evaluaton of a sphere
  /**
   * @return double the same evaluation than HFBlocks.
   */
  double eval (const Vector3D& position);

  
  // **********************
  // HyperFunable Interface
  // **********************
  
  void writeHFString (QTextStream & outStream);
  void writeDefArray (QTextStream & outStream);
  void writeDefBody (QTextStream & outStream);
 
  // *****************
  // Savable Interface
  // *****************
  
  void save (QTextStream & stream);
  void load (QTextStream & stream);

private:
  Vector3D d;
  Vector3D center;

};
  

#endif // BLock_H
