#ifndef CONVPOINTMANAGER_H
#define CONVPOINTMANAGER_H

#include <qvector.h>
#include "../Interfaces/PrimitiveManagerInterface.h"
#include <Object.h>
#include "../SurfelGenerator/SurfelGenerator.h"
#include "ConvPoint.h"

/// Events manager for ConvPoints.
/**
 * The ConvManager is able to create Convolutions surfaces based on points
 * with the UI events and the current Scene. 
 */
class ConvPointManager : public PrimitiveManagerInterface {
public :

  static ConvPointManager * getInstance ();
  static void destroyInstance ();

  PrimitiveInterface * getCurrentPrimitive (); 
  Object * getCurrentObject ();
  PrimitiveInterface * createPrimitive ();

  /**
   * Gets the <code>SurfelInterface</code> corresponding to the clicked position 
   * of the mouse from <code>FrameBufferInterface</code>, and begins the skeleton of 
   * the convolution.
   *
   * @param event the UI event gived by CoTeX.
   * @param confWidget the CoTeXConfigurationWidget, 
   *                   for quering options of creation (current texture, etc ...).
   *
   * @see #SurfelInterface
   * @see #FrameBufferInterface
   */
  void handleMousePressEvent (QMouseEvent *event, 
			      CoTeXConfigurationWidget * confWidget);

  /**
   * Handles the event of moving the mouse : each new point is added to the skeleton.
   * For optimisation, if the current point is too near to the last, it's not added to 
   * the skeleton (according to the value of the kernel).
   *
   * @param event the UI event gived by CoTeX.
   */
  void handleMouseMoveEvent (QMouseEvent *event);
  
  /**
   * Ends the creation of the ConvPoint.
   *
   * @see #SurfelInterface
   * @see #FrameBufferInterface
   */
  void handleMouseReleaseEvent (QMouseEvent *);
  
  /// Not useful for ConvPoint creation
  void handleKeyPressEvent (QKeyEvent *);

  /// Not useful for ConvPoint creation
  void handleKeyReleaseEvent (QKeyEvent *);

  
protected :

  /// Protected because of the design pattern <code>Singleton</code>
  ConvPointManager (); 
  
  /// Protected because of the design pattern <code>Singleton</code>
  virtual ~ConvPointManager ();
  

private :

  bool mousePressed;
  ConvPoint::ConvSkeleton * skeleton;
  
  ConvPoint * currentPrimitive; ///< the sphere currently constructed. 
  Object * currentPrimitiveObject; ///< the object representing the current instance.
  int instanceCount; ///< for generating unique names.
  CoTeXConfigurationWidget * configurationWidget;
  
  /// Destroy current non null resources. 
  void clean ();
};

#endif // CONVPOINTMANAGER_H
