#ifndef RBFMANAGER_H
#define RBFMANAGER_H

#include <Object.h>

#include "../Interfaces/PrimitiveManagerInterface.h"
#include "Rbf.h"

static const unsigned int RBF_SEQ_LEN = 4; ///< number of digit for Rbf number

/**
 * The RbfManager is able to create Rbf surfaces (Radial Basis Functions), based on
 * the UI events and the current Rbf.  
 *
 */
class RbfManager : public PrimitiveManagerInterface {
public :
  
  static RbfManager * getInstance ();
  static void destroyInstance ();

  PrimitiveInterface * getCurrentPrimitive (); 
  Object * getCurrentObject ();

  PrimitiveInterface * createPrimitive ();
  
  /**
   * It gets the <code>SurfelInterface</code> corresponding to the clicked position of the mouse
   * from <code>FrameBufferInterface</code>. If the mouse button pressed is the right button, it also
   * creates the <code>Rbf</code>.
   * The <code>Rbf</code> surface must be described by at least 3 points!!!!
   *
   * @param event the UI event gived by CoTeX.
   *
   * @see #SurfelInterface
   * @see #FrameBufferInterface
   * @see #QMouseEvent
   */
 
  void handleMousePressEvent (QMouseEvent *event, 
			      CoTeXConfigurationWidget * confWidget);
  /// Not useful for Rbf creation
  void handleMouseMoveEvent (QMouseEvent *) {}
  
  void handleMouseReleaseEvent (QMouseEvent *);
  
  /// Not useful for Rbf creation
  void handleKeyPressEvent (QKeyEvent *ke) {ke->ignore();}
  
  /// Not useful for Rbf creation
  void handleKeyReleaseEvent (QKeyEvent *ke) {ke->ignore();}
  

protected :
  
  RbfManager ();///<Constructor protected of the Rbf surface.
  virtual ~RbfManager ();
  

private :

  bool mousePressed;
  Rbf::RbfSet * rbfPointSet;///< the points describing the Rbf structure
  Rbf * currentPrimitive;///< the Rbf constructed by the RbfManager
  Object * currentPrimitiveObject;///< the PS object representing the current instance
  unsigned int instanceCount;///< for generating unique names.
  CoTeXConfigurationWidget * configurationWidget;


  /// Destroy current non-null resources. 
  void clean ();

  /// Draw base point of the rbf on the screen for easy vizualisation.
  void drawRbfPoints ();
};

#endif // RBFMANAGER_H

