#include <assert.h>
#include <math.h>

#include "Sphere.h"
static const double SPHERE_BBOX_SCALE = 1.2; ///< to ensure that the BBox will contains the entire sphere

Sphere::Sphere (QString n, const Vector3D& c, float r, TextureInterface * t)
  : PrimitiveInterface (n, t), radius (r), center (c) {
  assert (t != NULL && r >= 0.0);
}

Sphere::Sphere(Sphere &sphere) : PrimitiveInterface (sphere.getName (),
						     sphere.getTexture ()), 
				 radius (sphere.getRadius ()),
				 center (sphere.getCenter ()){
}

void Sphere::setRadius (float newRadius) { 
  assert (newRadius >= 0.0);
  radius = newRadius;
}

double Sphere::eval (const Vector3D& position) {
  return (radius - Vector3D::distance (position, center));  
}

BBoxMinMax Sphere::getBBoxMinMax () {
  BBoxMinMax bBoxMinMax = BBoxMinMax (center[0] - radius, center[1] - radius, 
				      center[2] - radius, center[0] + radius,
				      center[1] + radius, center[2] + radius);
  bBoxMinMax.scale (SPHERE_BBOX_SCALE);
  return bBoxMinMax;
}

// **********************
// HyperFunable Interface
// **********************

void Sphere::writeHFString (QTextStream & outStream) {
  outStream << "my_model(x[3],a[1],s[3])" << endl << "{" << endl;
  writeDefArray (outStream);
  writeDefBody (outStream);
  outStream << endl << "}" << endl;
}


void Sphere::writeDefArray (QTextStream & outStream) {
  outStream << "array center_" << getName () << "[3];" << endl;
  outStream << "array color_" << getName () << "[3];" << endl << endl;
}


void Sphere::writeDefBody (QTextStream & outStream) {
  QString center_name = QString ("center_") + getName ();
  QString radius_name = QString ("radius_") + getName ();
  QString color_name; 

  if (getFather () != NULL) 
    color_name = "color_" + getName ();
  else 
    color_name = "s";
  outStream << center_name << " = [" << center[0] << ", " << center[1] << ", " 
	    << center[2] << "];" << endl;
  outStream << radius_name << " = " << radius << ";" << endl ;
  outStream << color_name << " = ";
  getTexture ()->writeDefBody (outStream);
  outStream << ";" << endl;  
  outStream << getName () << " = hfSphere (x," << center_name << ", " << radius_name << ");" << endl << endl;
}


// *****************
// Savable Interface
// *****************

void Sphere::save (QTextStream &stream) {
  stream << "primitive sphere " <<  getName () << " " << center[0] 
	 << " " << center[1] << " " << center[2] << " "
	 << radius << " " << getTexture ()->getName ();
}

void Sphere::load (QTextStream &stream) {
  QString n;
  float x, y, z, r;
  stream >> n >> x >> y >> z >> r ;
  setName (n);
  center = Vector3D (x, y, z);
  radius = r;
}
