#include <math.h>
#include <assert.h>
#include "PerlinNoiseColorTexture.h"
#include "PerlinNoise.h"



PerlinNoiseColorTexture::PerlinNoiseColorTexture (QString n, 
						  ColorAttribute c1, 
						  ColorAttribute c2,
						  double pha,
						  double freq,
						  double amp,
						  double b1,
						  double b2,
						  NoiseType nType)
  : TextureInterface(n), color1 (c1), color2 (c2), phase (pha), frequency (freq), 
    amplitude (amp), bound1 (b1), bound2 (b2), noiseType (nType), 
    randomColor1 (false), randomColor2 (false) {}
  


const AttributeInterface * PerlinNoiseColorTexture::operator() (const Vector3D& pos){
  //double value = PerlinNoise::noise (pos[0], pos[1], pos[2]) + 1;
  double value = turbulence (pos[0], pos[1], pos[2]) + phase;
  switch (noiseType){
  case MARBLENOISE:{
    value = cos (pos[1] + value);
    break;
  }
    // different formules of post-process are used to obtain differents effects
  case WOODNOISE:
    value = (value * 14.0) - floor(value * 14.0);
    break;
  case CLOUDNOISE:
    value = 2.0*fabs(value) - 0.55;
    break;
  case V_SCRATCHED:
    value = cos(pos[0] + value);
    break;
  case H_SCRATCHED:
    value = cos(pos[2] + value);
    break;
  default:
    value = sin ((pos[0] * 7.0 + value * 9.0) * M_PI);
    break;
  }
  if(randomColor1){
    setColor1((int)rand()%255,(int)rand()%255,(int)rand()%255);
  }
  if(randomColor2){
    setColor2((int)rand()%255,(int)rand()%255,(int)rand()%255);
  }
  value = value * (amplitude/10) + (1 - (amplitude/10));
  
  mix (value);
  return &returnedColor;
}

double PerlinNoiseColorTexture::turbulence (float x, float y, float z){
  double t = 0.0;
  double freq = frequency;
  
  for (unsigned int i = 0; freq > 1; i++){
    t = t + fabs(PerlinNoise::noise (freq * x, freq * y, freq * z)) / freq;
    freq = freq/2;
  }
  return t;
}


void PerlinNoiseColorTexture::mix (double val){
  if (val > bound2) {
    returnedColor.setRgb (color2.rgb ());
  }
  else if (val < bound1){
    returnedColor.setRgb (color1.rgb ());
  }
  else {
    int red = (int)(color1.red()*val + (2 - val)*color2.red())/2;
    int green = (int)(color1.green()*val + (2 - val)*color2.green())/2;
    int blue = (int)(color1.blue()*val + (2 - val)*color2.blue())/2;
    returnedColor.setRgb (red, green, blue);
  }
}

// **********************
// Interface HyperFunable
// **********************

void PerlinNoiseColorTexture::writeHFString (QTextStream &) {
  
}

void PerlinNoiseColorTexture::writeDefArray (QTextStream &) {

}

void PerlinNoiseColorTexture::writeDefBody (QTextStream & outStream) {
  QString tmp = QString("hfA_NoiseP(x,%1,%2,%3,%4,%5)").arg (bound2).arg (bound1).arg (amplitude).arg (frequency).arg (phase);
  
  outStream << "[" << tmp << ", " << endl
	    << tmp << ", " << endl 
	    << tmp << "]";
}


// *****************
// Interface Savable
// *****************
  
void PerlinNoiseColorTexture::save (QTextStream & stream) {
  stream << "PerlinNoiseColorTexture " << getName () << " " << color1.red () 
	 << " " << color1.green () << " " << color1.blue () << " " << color2.red () 
	 << " " << color2.green () << " " << color2.blue () << " " << phase << " " 
	 << frequency << " " << amplitude << " " << bound1 << " " << bound2 << " " 
	 << noiseType;
}


void PerlinNoiseColorTexture::load (QTextStream & stream) {
  QString n;
  int r1,g1,b1,r2,g2,b2,nT;
  
  stream >> n >> r1 >> g1 >> b1 >> r2 >> g2 >> b2 ;
  stream >> phase >> frequency >> amplitude >> bound1 >> bound2 >> nT ;
  setName (n);
  color1.setRgb(r1, g1, b1);
  color2.setRgb(r2, g2, b2);
  noiseType = (NoiseType) nT;
}  
