#ifndef SIMPLECOLORTEXTURE_H
#define SIMPLECOLORTEXTURE_H

#include "../Interfaces/TextureInterface.h"
#include "../Attributes/ColorAttribute.h"

/// Uniforme RGB value in the space.
/**
 * Compute the same color for all the space.
 */
class SimpleColorTexture : public TextureInterface {
public :
  
  /// The base constructor , which need a ColorAttribute.
  /**
   * @param n the name.
   * @param c the color returned by this texture.
   */
  SimpleColorTexture (QString n = QString ("default"), 
		      ColorAttribute c = ColorAttribute ()) : TextureInterface (n), color (c) {}
  
  /// Constructoir with direct RGB value of the base color.
  SimpleColorTexture (QString n, int red, int green, int blue);

  virtual ~SimpleColorTexture () {}
  
  inline ColorAttribute getColor () const { return color;}
  inline void setColor (ColorAttribute & newColor) { color = newColor;} 

  // **************************
  // Interface TextureInterface
  // **************************
  const AttributeInterface * operator() (const Vector3D&) { return &color; }

  
  // **********************
  // Interface HyperFunable
  // **********************
  
  void writeHFString (QTextStream & outStream);
  void writeDefArray (QTextStream & outStream);
  void writeDefBody (QTextStream & outStream);
  
  // *****************
  // Interface Savable
  // *****************
  
  void save (QTextStream & stream);
  void load (QTextStream & stream);

private :

  ColorAttribute color;

};

/// For easy primitive creation.
extern SimpleColorTexture * DEFAULT_SIMPLECOLOR;

#endif // SIMPLECOLORTEXTURE_H
