#include <qlabel.h>
#include <qfont.h>
#include <qiconset.h>
#include <qcolordialog.h>
#include "AbstractConfigurationWidget.h"

static const QSize TEX_WIDGET_SIZE = QSize (220, 150);
static const QSize TEX_COLORBUTTON_SIZE = QSize (20, 20);

AbstractConfigurationWidget::AbstractConfigurationWidget (QWidget * parent)
  : QGroupBox (parent) {

  resizeWidget (this, TEX_WIDGET_SIZE);
  
  QFont font (this->font ());
  font.setPointSize (COTEX_FONT_SIZE);
  this->setFont (font);

  QLabel * nameLabel = new QLabel ("Name : ", this);
  nameLabel->setMinimumSize (nameLabel->sizeHint ());

  nameLineEdit = new QLineEdit (TEX_BASENAME, this);
  nameLineEdit->setMinimumSize (nameLineEdit->sizeHint ());
  
  baseLayout = new QHBoxLayout ();
  baseLayout->setMargin (TEX_MARGIN);
  baseLayout->setSpacing (TEX_SPACING);
  baseLayout->addWidget (nameLabel);
  baseLayout->addWidget (nameLineEdit);
}

// ********
// Services
// ********

QPushButton * AbstractConfigurationWidget::createColorChooserButton (const QString & name, 
								     const QColor & baseColor) {
  QPushButton * colorChooserButton = createPushButton (name, this, TEX_COLORBUTTON_SIZE);
  updateButtonColor (colorChooserButton, baseColor);
  return colorChooserButton;
}

void AbstractConfigurationWidget::updateButtonColor (QPushButton * button, const QColor & color) {
  QPixmap pix ((TEX_COLORBUTTON_SIZE.width ()) / 2, (TEX_COLORBUTTON_SIZE.height ()) / 2);
  pix.fill (color);
  button->setIconSet (QIconSet (pix));
}

void AbstractConfigurationWidget::selectColor (QPushButton * button, 
					       QColor & color) {
  color = QColorDialog::getColor (color, this);
  updateButtonColor (button, color);
}

QString AbstractConfigurationWidget::getCurrentTextureName () const{
  return nameLineEdit->text ();
}

void AbstractConfigurationWidget::setCurrentTextureName (QString & newName) {
  nameLineEdit->setText (newName);
}

QHBoxLayout * AbstractConfigurationWidget::getBaseLayout () const {
  return baseLayout;
}

void AbstractConfigurationWidget::resizeWidget (QWidget * widget, QSize size) {
  widget->resize (size);
  widget->setMinimumSize (size);
  widget->setMaximumSize (size);
}

QPushButton * AbstractConfigurationWidget::createPushButton (QString label,
							     QWidget * parent,
							     QSize size) {
  QPushButton * button = new QPushButton (label, parent);
  resizeWidget(button, size);
  return button;
}
