#ifndef COTEXCONFIGURATIONWIDGET_H
#define COTEXCONFIGURATIONWIDGET_H

#include <qlistbox.h>
#include <qpixmap.h>
#include <qwidgetstack.h>

#include "TextureConfigurationWidget.h"
#include "ConvConfigurationWidget.h"
#include "RBFConfigurationWidget.h"
#include "../src/Textures/PerlinNoiseColorTexture.h"
#include "../src/Textures/SimpleColorTexture.h"

/// Class for the widget of configuration of CoTeX
/**
 * This widget allows to do the settings of the CoTeX tool.
 */ 
class CoTeXConfigurationWidget : public QWidget { 

  Q_OBJECT
  
public:

  /// The constructor
  /**
   * @param tMap is the only one obligatory, it is the registration 
   * of all the textures use in the currrent session of PointShop.
   * It will be increased by the TextureConfigurationWidget.
   */ 
  CoTeXConfigurationWidget (TextureInterface::TextureMap * tMap, 
			    QWidget * parent = 0, const char * name = 0, 
			    WFlags fl = 0);

  /// The destructor
  virtual ~CoTeXConfigurationWidget () {}
  
  // ********
  // Services
  // ********
  
  /// NULL if the textured named <code>textureName>/code> doesn't appear in the list.
  TextureInterface * getTexture (QString textureName) const;
  inline TextureInterface * getCurrentTexture () const { return (*texMap)[texturesListBox->currentText ()];}
  inline QString getCurrentPrimitiveTypeName () const { return primitivesListBox->currentText ();}
  inline QString getCurrentOperatorTypeName () const { return operatorsListBox->currentText ();}
  inline double getConvS () const { return convConfigurationWidget->getS ();}
  inline double getConvThreshold () const { return convConfigurationWidget->getThreshold ();}
  inline double getRBFThreshold () const { return rbfConfigurationWidget->getThreshold ();}
  
  /// Insert an existing SimpeColorTexture in the current list.
  void insertTexture (SimpleColorTexture * tex);

  /// Insert an existing PerlinNoiseColorTexture in the current list.
  void insertTexture (PerlinNoiseColorTexture * tex);

signals :

  void loadSignal (QString filename);
  void saveSignal (QString filename);
  void exportHFSignal (QString filename);
  void resetSignal ();

private:

  TextureConfigurationWidget * textureConfigurationWidget; ///< the widget for the creation of new textures
  ConvConfigurationWidget * convConfigurationWidget; ///< the widget for configuring the convolution
  RBFConfigurationWidget * rbfConfigurationWidget; ///< the widget for configuring the RBF
  TextureInterface::TextureMap * texMap; ///< the map of the existing textures 
  
  QListBox * texturesListBox; 
  QListBox * primitivesListBox;
  QListBox * operatorsListBox;
  QWidgetStack * primitivesWidgetStack;

  /// Map for quering configuration Widget of the primitives with their id.
  typedef QMap<int,QWidget*> ConfigurationWidgetMap;
  ConfigurationWidgetMap configurationWidgetMap;

  /// Resizes the widget to a given size  
  void resizeWidget (QWidget *, QSize);

  /// create a QPushButton at the given size 
  QPushButton * createPushButton (QString, QWidget *, QSize);
  
  /// Places some default textures
  void defaultTextures ();
  
private slots :

  void handleLoad ();
  void handleSave ();
  void handleExportHF ();

  void appearTextureWidget ();
  void addNewTextureName (QString, QPixmap);
  void primitiveDisplay (int);
};

#endif // COTEXCONFIGURATIONWIDGET_H
